@@ -1,18 +1,3 @@
-%% Copyright (C) 2009 Carlo de Falco
-%% 
-%% This program is free software; you can redistribute it and/or modify
-%% it under the terms of the GNU General Public License as published by
-%% the Free Software Foundation; either version 2 of the License, or
-%% (at your option) any later version.
-%% 
-%% This program is distributed in the hope that it will be useful,
-%% but WITHOUT ANY WARRANTY; without even the implied warranty of
-%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-%% GNU General Public License for more details.
-%% 
-%% You should have received a copy of the GNU General Public License
-%% along with this program; if not, see <http://www.gnu.org/licenses/>.
-
 function varargout = nrbbasisfunder (points, nrb)
 
 % NRBBASISFUNDER:  NURBS basis functions derivatives
@@ -45,6 +30,20 @@ function varargout = nrbbasisfunder (points, nrb)
 %      N - Indices of the basis functions that are nonvanishing at each
 %          point. size(N) == size(B)
 %   
+%    Copyright (C) 2009 Carlo de Falco
+%
+%    This program is free software: you can redistribute it and/or modify
+%    it under the terms of the GNU General Public License as published by
+%    the Free Software Foundation, either version 3 of the License, or
+%    (at your option) any later version.
+
+%    This program is distributed in the hope that it will be useful,
+%    but WITHOUT ANY WARRANTY; without even the implied warranty of
+%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+%    GNU General Public License for more details.
+%
+%    You should have received a copy of the GNU General Public License
+%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
   
   if (   (nargin<2) ...
@@ -54,14 +53,14 @@ function varargout = nrbbasisfunder (points, nrb)
       || (~iscell(points) && iscell(nrb.knots) && (size(points,1)~=2)) ...
       || (~iscell(nrb.knots) && (nargout>2)) ...
       )
-    print_usage();
+    error('Incorrect input arguments in nrbbasisfun');
   end
                             
-  if (~iscell(nrb.knots))    %% NURBS curve
+  if (~iscell(nrb.knots))         %% NURBS curve
     
     [varargout{1}, varargout{2}] = nrb_crv_basisfun_der__ (points, nrb);
 
-  else                       %% NURBS surface
+  elseif size(nrb.knots,2) == 2 %% NURBS surface
 
     if (iscell(points))
       [v, u] = meshgrid(points{2}, points{1});
@@ -72,39 +71,22 @@ function varargout = nrbbasisfunder (points, nrb)
     
     [varargout{1}, varargout{2}, varargout{3}] = nrb_srf_basisfun_der__ (p, nrb);
 
+  else                            %% NURBS volume
+    error('The function nrbbasisfunder is not yet ready for volumes')
   end
 end
-
-  function [Bu, nbfu] = nrb_crv_basisfun_der__ (points, nrb);
-    n    = size (nrb.coefs, 2) -1;
-    p    = nrb.order -1;
-    u    = points;
-    U    = nrb.knots;
-    w    = nrb.coefs(4,:);
-    
-    spu  =  findspan (n, p, u, U);
-    nbfu =  numbasisfun (spu, u, p, U);
-    
-    N     = basisfun (spu, u, p, U);
-
-    Nprime = basisfunder (spu, p, u, U, 1);
-    Nprime = squeeze(Nprime(:,2,:));
-
-    
-    [Dpc, Dpk]  = bspderiv (p, w, U);
-    D           = bspeval  (p, w, U, u);
-    Dprime      = bspeval  (p-1, Dpc, Dpk, u);
-    
-
-    Bu1   = bsxfun (@(np, d) np/d , Nprime.', D);
-    Bu2   = bsxfun (@(n, dp)  n*dp, N.', Dprime./D.^2);
-    Bu    = w(nbfu+1) .* (Bu1 - Bu2).';
-
-  end
-
- 
-  
   
+%!demo
+%! U = [0 0 0 0 1 1 1 1];
+%! x = [0 1/3 2/3 1] ;
+%! y = [0 0 0 0];
+%! w = [1 1 1 1];
+%! nrb = nrbmak ([x;y;y;w], U);
+%! u = linspace(0, 1, 30);
+%! [Bu, id] = nrbbasisfunder (u, nrb);
+%! plot(u, Bu)
+%! title('Derivatives of the cubic Bernstein polynomials')
+%! hold off
 
 %!test
 %! U = [0 0 0 0 1 1 1 1];
@@ -132,9 +114,10 @@ end
 %! Lx  = 1; Ly  = 1; 
 %! nrb = nrb4surf   ([0 0], [1 0], [0 1], [1 1]);
 %! nrb = nrbdegelev (nrb, [p-1, q-1]);
-%! nrb = nrbkntins  (nrb, {linspace(0,1,m)(2:end-1), linspace(0,1,n)(2:end-1)});
-%! nrb.coefs (4,:,:) += rand (size (nrb.coefs (4,:,:)));
-%! tic(); [Bu, Bv, N] = nrbbasisfunder ({rand(1, 20), rand(1, 20)}, nrb); toc()
+%! aux1 = linspace(0,1,m); aux2 = linspace(0,1,n);
+%! nrb = nrbkntins  (nrb, {aux1(2:end-1), aux2(2:end-1)});
+%! nrb.coefs (4,:,:) = nrb.coefs(4,:,:) + rand (size (nrb.coefs (4,:,:)));
+%! [Bu, Bv, N] = nrbbasisfunder ({rand(1, 20), rand(1, 20)}, nrb);
 %! #plot3(squeeze(u(1,:,:)), squeeze(u(2,:,:)), reshape(Bu(:,10), 20, 20),'o')
 %! assert (sum (Bu, 2), zeros(20^2, 1), 1e-10)
 
